@@ -15,18 +15,26 @@
 plotroc = function( sim.obj, bdgraph.obj, bdgraph.obj2 = NULL, bdgraph.obj3 = NULL, 
                     bdgraph.obj4 = NULL, cut = 20, smooth = FALSE, label = TRUE, main = "ROC Curve" )
 {
-    if ( class( sim.obj ) == "sim" ) G = as.matrix( sim.obj $ G ) else G = as.matrix( sim.obj )
-	G[ lower.tri( G, diag = TRUE ) ] = 0
+    if( is.matrix( sim.obj ) ) 
+    {
+        if( ( sum( sim.obj == 0 ) + sum( sim.obj == 1 ) ) != ( nrow( sim.obj ) ^ 2 ) ) stop( "Element of 'sim.obj' must be 0 or 1" )
+        G = sim.obj
+    }else{
+        if( class( sim.obj ) == "sim"   ) G <- unclass( sim.obj $ G ) 
+        if( class( sim.obj ) == "graph" ) G <- unclass( sim.obj ) 
+    }
+    
+    G[ lower.tri( G, diag = TRUE ) ] = 0
 	
     output_tp_fp = compute_tp_fp( G = G, bdgraph.obj = bdgraph.obj, cut = cut, smooth = smooth )
     fp           = output_tp_fp $ fp
     tp           = output_tp_fp $ tp
  	
-	# par( mar = c( 3.8, 4.2, 1.8, 1 ) )
-    plot( NA, type = "l", col = "black", cex.lab = 1.3, cex.main = 2, cex.axis = 1.2,
+	# graphics::par( mar = c( 3.8, 4.2, 1.8, 1 ) )
+    graphics::plot( NA, type = "l", col = "black", cex.lab = 1.3, cex.main = 2, cex.axis = 1.2,
           main = main, xlab = "False Postive Rate", ylab = "True Postive Rate", 
           ylim = c( 0, 1 ), xlim = c( 0, 1 ) )
-    points( x = fp, y = tp, type = "l", col = "black", lty = 1, lw = 2 )
+    graphics::points( x = fp, y = tp, type = "l", col = "black", lty = 1, lw = 2 )
   
     if( !is.null( bdgraph.obj2 ) )
     {
@@ -34,7 +42,7 @@ plotroc = function( sim.obj, bdgraph.obj, bdgraph.obj2 = NULL, bdgraph.obj3 = NU
 		fp_2         = output_tp_fp $ fp
 		tp_2         = output_tp_fp $ tp
 	
-        points( x = fp_2, y = tp_2, type = "l", col = "blue", lty = 2, lw = 2 )
+        graphics::points( x = fp_2, y = tp_2, type = "l", col = "blue", lty = 2, lw = 2 )
     }
     
     if( !is.null( bdgraph.obj3 ) )
@@ -43,7 +51,7 @@ plotroc = function( sim.obj, bdgraph.obj, bdgraph.obj2 = NULL, bdgraph.obj3 = NU
 		fp_3         = output_tp_fp $ fp
 		tp_3         = output_tp_fp $ tp
    		
-        points( x = fp_3, y = tp_3, type = "l", col = "green", lty = 3, lw = 2 )
+        graphics::points( x = fp_3, y = tp_3, type = "l", col = "green", lty = 3, lw = 2 )
     }
     
     if( !is.null( bdgraph.obj4 ) )
@@ -52,14 +60,18 @@ plotroc = function( sim.obj, bdgraph.obj, bdgraph.obj2 = NULL, bdgraph.obj3 = NU
 		fp_4         = output_tp_fp $ fp
 		tp_4         = output_tp_fp $ tp
    		
-        points( x = fp_4, y = tp_4, type = "l", col = "red", lty = 4, lw = 2 )
+        graphics::points( x = fp_4, y = tp_4, type = "l", col = "red", lty = 4, lw = 2 )
     }
     
-	if ( label )
+	if( ( length( label ) == 1 ) && ( label == TRUE ) )
 	{ 
-		if( !is.null( bdgraph.obj2 ) && is.null( bdgraph.obj3 ) )  legend( "bottomright", c( "bdgraph.obj", "bdgraph.obj2" ), lty = 1:2, col = c( "black", "blue" ), lwd = c( 2, 2 ), cex = 1.5 )
-		if( !is.null( bdgraph.obj3 ) && is.null( bdgraph.obj4 ) )  legend( "bottomright", c( "bdgraph.obj", "bdgraph.obj2", "bdgraph.obj3" ), lty = 1:3, col = c( "black", "blue", "green" ), lwd = c( 2, 2 ), cex = 1.5 )
-		if( !is.null( bdgraph.obj4 ) ) legend( "bottomright", c( "bdgraph.obj", "bdgraph.obj2", "bdgraph.obj3", "bdgraph.obj4" ), lty = 1:4, col = c( "black", "blue", "green", "red" ), lwd = c( 2, 2 ), cex = 1.5 )
+		if( !is.null( bdgraph.obj2 ) && is.null( bdgraph.obj3 ) )  graphics::legend( "bottomright", c( "bdgraph.obj", "bdgraph.obj2" ), lty = 1:2, col = c( "black", "blue" ), lwd = c( 2, 2 ), cex = 1.5 )
+		if( !is.null( bdgraph.obj3 ) && is.null( bdgraph.obj4 ) )  graphics::legend( "bottomright", c( "bdgraph.obj", "bdgraph.obj2", "bdgraph.obj3" ), lty = 1:3, col = c( "black", "blue", "green" ), lwd = c( 2, 2 ), cex = 1.5 )
+		if( !is.null( bdgraph.obj4 ) ) graphics::legend( "bottomright", c( "bdgraph.obj", "bdgraph.obj2", "bdgraph.obj3", "bdgraph.obj4" ), lty = 1:4, col = c( "black", "blue", "green", "red" ), lwd = c( 2, 2 ), cex = 1.5 )
+	}else if( label != FALSE ){
+	    if( !is.null( bdgraph.obj2 ) && is.null( bdgraph.obj3 ) )  graphics::legend( "bottomright", label, lty = 1:2, col = c( "black", "blue" ), lwd = c( 2, 2 ), cex = 1.5 )
+	    if( !is.null( bdgraph.obj3 ) && is.null( bdgraph.obj4 ) )  graphics::legend( "bottomright", label, lty = 1:3, col = c( "black", "blue", "green" ), lwd = c( 2, 2 ), cex = 1.5 )
+	    if( !is.null( bdgraph.obj4 ) ) graphics::legend( "bottomright", label, lty = 1:4, col = c( "black", "blue", "green", "red" ), lwd = c( 2, 2 ), cex = 1.5 )
 	}   
 }
        
@@ -73,23 +85,16 @@ compute_tp_fp = function( G, bdgraph.obj, cut, smooth )
 	sum_edges   = sum( upper_G )
 	sum_no_dges = p * ( p - 1 ) / 2 - sum_edges
 
-	if( class( bdgraph.obj ) == "bdgraph" )
+	if( ( class( bdgraph.obj ) == "bdgraph" ) | ( class( bdgraph.obj ) == "ssgraph" ) )
 	{
 	    p_links = bdgraph.obj $ p_links
 	    if( is.null( p_links ) ) p_links = BDgraph::plinks( bdgraph.obj, round = 10 )
 	    p_links = as.matrix( p_links )
 	}
 	
-	if( class( bdgraph.obj ) == "ssgraph" )
-	{
-	    p_links = bdgraph.obj $ p_links
-	    if( is.null( p_links ) ) p_links = BDgraph::plinks( bdgraph.obj, round = 10 )
-	    p_links = as.matrix( p_links )
-	}
-
 	if( is.matrix( bdgraph.obj ) )
 	{
-	    if( any( bdgraph.obj < 0 ) || any( bdgraph.obj > 1 ) ) stop( "Values of 'bdgraph.obj' must be between ( 0, 1 )." )
+	    if( any( bdgraph.obj < 0 ) || any( bdgraph.obj > 1 ) ) stop( " Values of 'bdgraph.obj' must be between ( 0, 1 )." )
 	    p_links = as.matrix( bdgraph.obj )
 	}
 	
@@ -130,7 +135,7 @@ compute_tp_fp = function( G, bdgraph.obj, cut, smooth )
 	
 	if ( smooth == TRUE )
 	{
-		fit = smooth.spline( x = fp, y = tp )
+		fit = stats::smooth.spline( x = fp, y = tp )
 		fp  = c( 0, fit $ x )
 		tp  = c( 0, fit $ y )
 	}	
