@@ -29,7 +29,7 @@ select = function( bdgraph.obj, cut = NULL, vis = FALSE )
     
 	if( is.matrix( bdgraph.obj ) ) 
 	{
-	    if( any( bdgraph.obj < 0 ) || any( bdgraph.obj > 1 ) ) stop( "Values of 'bdgraph.obj' must be between ( 0, 1 )." )
+	    if( any( bdgraph.obj < 0 ) || any( bdgraph.obj > 1 ) ) stop( " Values of 'bdgraph.obj' must be between ( 0, 1 )." )
 	    p_links = bdgraph.obj
 		p       = nrow( p_links )
 	}
@@ -67,15 +67,15 @@ select = function( bdgraph.obj, cut = NULL, vis = FALSE )
 		
 	if( vis )
 	{
-		G <- graph.adjacency( selected_g, mode = "undirected", diag = FALSE )
+		G <- igraph::graph.adjacency( selected_g, mode = "undirected", diag = FALSE )
 		if( p < 20 ) sizev = 15 else sizev = 2
 
 		if( is.null( cut ) )
 		{
-			plot.igraph( G, layout = layout.circle, main = "Graph with highest posterior probability", sub = paste( c( "Posterior probability = ", round( max( graph_weights ) / sum( graph_weights ), 4) ), collapse = "" ),
+			igraph::plot.igraph( G, layout = igraph::layout.circle, main = "Graph with highest posterior probability", sub = paste( c( "Posterior probability = ", round( max( graph_weights ) / sum( graph_weights ), 4) ), collapse = "" ),
 			            vertex.color = "white", vertex.size = sizev, vertex.label.color = 'black'  )
 		}else{
-			plot.igraph( G, layout = layout.circle, main = paste( c( "Graph with links posterior probabilities > ",  cut ), collapse = "" ), vertex.color = "white", vertex.size = sizev, vertex.label.color = 'black' )
+			igraph::plot.igraph( G, layout = igraph::layout.circle, main = paste( c( "Graph with links posterior probabilities > ",  cut ), collapse = "" ), vertex.color = "white", vertex.size = sizev, vertex.label.color = 'black' )
 		}
 	}
 
