@@ -49,10 +49,13 @@ void rwish_c( double Ts[], double K[], int *b, int *p )
 // ------------------------------------------------------------------------------------------------|
 // G is adjacency matrix which has zero in its diagonal // threshold = 1e-8
 // ------------------------------------------------------------------------------------------------|
-void rgwish_c( int G[], double Ts[], double K[], int *b, int *p )
+void rgwish_c( int G[], double Ts[], double K[], int *b, int *p, double *threshold )
 {
 	int info, i, j, l, size_node, one = 1, dim = *p, pxp = dim * dim;	
-	double alpha = 1.0, beta  = 0.0, temp, threshold = 1e-8;
+    
+    double threshold_c = *threshold;
+	double alpha = 1.0, beta  = 0.0;
+	
 	char transN  = 'N', uplo  = 'U'; 
 	
 	rwish_c( Ts, K, b, &dim );
@@ -69,8 +72,9 @@ void rgwish_c( int G[], double Ts[], double K[], int *b, int *p )
 	vector<int> N_i( dim );                  // For dynamic memory used
 	vector<double> sigma_N_i( pxp );         // For dynamic memory used
 
-	double max_diff = 1.0;	
-	while( max_diff > threshold )
+//	double temp, max_diff = 1.0;
+    double mean_diff = 1.0;
+	while( mean_diff > threshold_c )
 	{
 		memcpy( &sigma_last[0], &sigma[0], sizeof( double ) * pxp );
 		
@@ -78,20 +82,20 @@ void rgwish_c( int G[], double Ts[], double K[], int *b, int *p )
 		{
 			// Count size of note
 			size_node = 0;
-			for( j = 0; j < dim; j++ ) size_node += G[j * dim + i];
+			for( j = 0; j < dim; j++ ) size_node += G[ j * dim + i ];
 
 			if( size_node > 0 )
 			{
 				l = 0;
 				for( j = 0; j < dim; j++ )
 				{
-					if( G[j * dim + i] )
+					if( G[ j * dim + i ] )
 					{
-						sigma_start_N_i[l] = sigma_start[i * dim + j]; 
-						N_i[l++]           = j;
+						sigma_start_N_i[ l ] = sigma_start[ i * dim + j ]; 
+						N_i[ l++ ]           = j;
 					}
 					else
-						beta_star[j] = 0.0; 
+						beta_star[ j ] = 0.0; 
 				}
 				// --------------------------------------------------------------------------------|
 				
@@ -100,44 +104,47 @@ void rgwish_c( int G[], double Ts[], double K[], int *b, int *p )
 				// A * X = B   for   sigma_start_N_i := (sigma_N_i)^{-1} * sigma_start_N_i
 				F77_NAME(dposv)( &uplo, &size_node, &one, &sigma_N_i[0], &size_node, &sigma_start_N_i[0], &size_node, &info );
 
-				for( j = 0; j < size_node; j++ ) beta_star[N_i[j]] = sigma_start_N_i[j];
+				for( j = 0; j < size_node; j++ ) beta_star[ N_i[ j ] ] = sigma_start_N_i[ j ];
 				
 				F77_NAME(dgemm)( &transN, &transN, &dim, &one, &dim, &alpha, &sigma[0], &dim, &beta_star[0], &dim, &beta, &sigma_start_i[0], &dim );
 				
 				for( j = 0; j < i; j++ )
 				{
-					sigma[j * dim + i] = sigma_start_i[j];
-					sigma[i * dim + j] = sigma_start_i[j];
+					sigma[ j * dim + i ] = sigma_start_i[ j ];
+					sigma[ i * dim + j ] = sigma_start_i[ j ];
 				}
 				
 				for( j = i + 1; j < dim; j++ )
 				{
-					sigma[j * dim + i] = sigma_start_i[j];
-					sigma[i * dim + j] = sigma_start_i[j];
+					sigma[ j * dim + i ] = sigma_start_i[ j ];
+					sigma[ i * dim + j ] = sigma_start_i[ j ];
 				}
-			} 
-			else 
-			{
+			}else{
 				for( j = 0; j < i; j++ )
 				{
-					sigma[j * dim + i] = 0.0;
-					sigma[i * dim + j] = 0.0;
+					sigma[ j * dim + i ] = 0.0;
+					sigma[ i * dim + j ] = 0.0;
 				}
 				
 				for( j = i + 1; j < dim; j++ )
 				{
-					sigma[j * dim + i] = 0.0;
-					sigma[i * dim + j] = 0.0;
+					sigma[ j * dim + i ] = 0.0;
+					sigma[ i * dim + j ] = 0.0;
 				}
 			} 
 		}
 
-		max_diff = fabs( static_cast<double>( sigma[0] - sigma_last[0] ) );
+		mean_diff = fabs( static_cast<double>( sigma[0] - sigma_last[0] ) );
 		for( i = 1; i < pxp; i++ )
-		{
-			temp = fabs( static_cast<double>( sigma[i] - sigma_last[i] ) );
-			if( temp > max_diff ) max_diff = temp; 
-		}		
+		    mean_diff += fabs( static_cast<double>( sigma[ i ] - sigma_last[ i ] ) );
+		mean_diff /= pxp;
+
+//		max_diff = fabs( static_cast<double>( sigma[0] - sigma_last[0] ) );
+//		for( i = 1; i < pxp; i++ )
+//		{
+//			temp = fabs( static_cast<double>( sigma[i] - sigma_last[i] ) );
+//			if( temp > max_diff ) max_diff = temp; 
+//		}		
 	}
 
 	inverse( &sigma[0], K, &dim );
@@ -146,12 +153,15 @@ void rgwish_c( int G[], double Ts[], double K[], int *b, int *p )
 // ------------------------------------------------------------------------------------------------|
 // rgwish ONLY for inside of MCMC algorithm
 // ------------------------------------------------------------------------------------------------|
-void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sigma[], int *b_star, int *p,
+void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sigma[], int *b_star, 
+					int *p, double *threshold,
 					double sigma_start[], double inv_C[], double beta_star[], double sigma_i[], 
 					vector<double> &sigma_start_N_i, vector<double> &sigma_N_i, vector<int> &N_i )
 {
 	int i, i1, j, ij, ip, l, size_node_i, info, one = 1, dim = *p, pxp = dim * dim, dim1 = dim + 1, bKdim = *b_star + dim - 1;	
-	double temp, threshold = 1e-8, alpha = 1.0, beta  = 0.0, max_diff = 1.0;	
+	
+	double alpha = 1.0, beta  = 0.0;	
+	
 	char transT  = 'T', transN = 'N', side = 'R', upper = 'U';																	
 	
 	// STEP 1: sampling from wishart distributions
@@ -159,7 +169,7 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 	//GetRNGstate();
 	#pragma omp parallel for
 	for( i = 0; i < dim; i++ )
-		sigma_start[i * dim1] = sqrt( Rf_rgamma( ( bKdim - i ) / 2.0, 2.0 ) ); // i * dim1 = i * dim + i
+		sigma_start[ i * dim1 ] = sqrt( Rf_rgamma( ( bKdim - i ) / 2.0, 2.0 ) ); // i * dim1 = i * dim + i
 		//sigma_start[i * dim1] = sqrt( rchisq( bKdim - i ) ); // i * dim1 = i * dim + i
 
 	#pragma omp parallel for
@@ -180,7 +190,7 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 	#pragma omp parallel for
 	for( i = 0; i < dim; i++ )
 		for( int j = 0; j < dim; j++ )
-			inv_C[j * dim + i] = ( i == j );	
+			inv_C[ j * dim + i ] = ( i == j );	
 	
 	// op( A )*X = alpha*B,   or   X*op( A ) = alpha*B,
 	F77_NAME(dtrsm)( &side, &upper, &transN, &transN, &dim, &dim, &alpha, &sigma_start[0], &dim, &inv_C[0], &dim );
@@ -190,28 +200,32 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 
 	memcpy( sigma, &sigma_start[0], sizeof( double ) * pxp ); 
 	
-	while ( max_diff > threshold )
+//	double temp, max_diff = 1.0, threshold_c = *threshold;
+	double mean_diff = 1.0, threshold_c = *threshold;
+int counter = 0;
+	while( ( mean_diff > threshold_c ) and ( counter < 5000 ) )
 	{
-		max_diff = 0.0;
+		counter++;
+	    mean_diff = 0.0;
 		
 		for( i = 0; i < dim; i++ )
 		{
 			ip = i * dim;
 
-			size_node_i = size_node[i];
+			size_node_i = size_node[ i ];
 			if( size_node_i > 0 )
 			{
 				l = 0;
 				for( j = 0; j < dim; j++ )
 				{
 					ij = ip + j;
-					if( G[ij] )
+					if( G[ ij ] )
 					{
-						sigma_start_N_i[l] = sigma_start[ij]; 
-						N_i[l++]           = j;
+						sigma_start_N_i[ l ] = sigma_start[ ij ]; 
+						N_i[ l++ ]           = j;
 					}
 					else
-						beta_star[j] = 0.0; 
+						beta_star[ j ] = 0.0; 
 				}
 				// --------------------------------------------------------------------------------|
 				
@@ -220,7 +234,7 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 				// A * X = B   for   sigma_start_N_i := (sigma_N_i)^{-1} * sigma_start_N_i
 				F77_NAME(dposv)( &upper, &size_node_i, &one, &sigma_N_i[0], &size_node_i, &sigma_start_N_i[0], &size_node_i, &info );
 
-				for( j = 0; j < size_node_i; j++ ) beta_star[N_i[j]] = sigma_start_N_i[j];
+				for( j = 0; j < size_node_i; j++ ) beta_star[ N_i[ j ] ] = sigma_start_N_i[ j ];
 	
 				// sigma_i = sigma %*% beta_star
 				F77_NAME(dsymv)( &side, &dim, &alpha, sigma, &dim, &beta_star[0], &one, &beta, &sigma_i[0], &one );
@@ -229,11 +243,12 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 				
 				for( j = 0; j < i; j++ )
 				{
-					ij        = j * dim + i;
-					temp      = fabs( static_cast<double>( sigma[ij] - sigma_i[j] ) );
-					max_diff  = ( temp > max_diff ) ? temp : max_diff; 					
+					ij         = j * dim + i;
+					mean_diff += fabs( static_cast<double>( sigma[ ij ] - sigma_i[ j ] ) );
+//					temp      = fabs( static_cast<double>( sigma[ ij ] - sigma_i[ j ] ) );
+//					max_diff  = ( temp > max_diff ) ? temp : max_diff; 					
 
-					sigma[ij] = sigma_i[j];
+					sigma[ ij ] = sigma_i[ j ];
 				}
 				
 				i1 = i + 1;
@@ -241,36 +256,39 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 
 				for( j = i1; j < dim; j++ )
 				{
-					ij        = j * dim + i;
-					temp      = fabs( static_cast<double>( sigma[ij] - sigma_i[j] ) );
-					max_diff  = ( temp > max_diff ) ? temp : max_diff; 					
+					ij         = j * dim + i;
+					mean_diff += fabs( static_cast<double>( sigma[ ij ] - sigma_i[ j ] ) );
+//					temp      = fabs( static_cast<double>( sigma[ ij ] - sigma_i[ j ] ) );
+//					max_diff  = ( temp > max_diff ) ? temp : max_diff; 					
 
-					sigma[ij] = sigma_i[j];
+					sigma[ ij ] = sigma_i[ j ];
 				}
-			} 
-			else 
-			{					
+			}else{					
 				for( j = 0; j < i; j++ )
 				{
-					ij       = j * dim + i;
-					temp     = fabs( static_cast<double>( sigma[ij] ) );
-					max_diff = ( temp > max_diff ) ? temp : max_diff; 					
+					ij         = j * dim + i;
+				    mean_diff += fabs( static_cast<double>( sigma[ ij ] ) );
+//				    temp     = fabs( static_cast<double>( sigma[ ij ] ) );
+//					max_diff = ( temp > max_diff ) ? temp : max_diff; 					
 
-					sigma[ij]     = 0.0;
-					sigma[ip + j] = 0.0;
+					sigma[ ij ]     = 0.0;
+					sigma[ ip + j ] = 0.0;
 				}
 				
 				for( j = i + 1; j < dim; j++ )
 				{
-					ij       = j * dim + i;
-					temp     = fabs( static_cast<double>( sigma[ij] ) );
-					max_diff = ( temp > max_diff ) ? temp : max_diff; 					
+					ij         = j * dim + i;
+					mean_diff += fabs( static_cast<double>( sigma[ ij ] ) );
+//					temp     = fabs( static_cast<double>( sigma[ ij ] ) );
+//					max_diff = ( temp > max_diff ) ? temp : max_diff; 					
 
-					sigma[ij]     = 0.0;
-					sigma[ip + j] = 0.0;				
+					sigma[ ij ]     = 0.0;
+					sigma[ ip + j ] = 0.0;				
 				}
 			} 
 		}
+		
+		mean_diff /= pxp;
 	}
 	
 	memcpy( &sigma_start[0], sigma, sizeof( double ) * pxp );	 	
@@ -280,7 +298,7 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 	#pragma omp parallel for
 	for( i = 0; i < dim; i++ )
 		for( int j = 0; j < dim; j++ )
-			K[j * dim + i] = ( i == j );
+			K[ j * dim + i ] = ( i == j );
 	
 	// LAPACK function: computes solution to A * X = B, where A is symmetric positive definite matrix
 	F77_NAME(dposv)( &upper, &dim, &dim, &sigma_start[0], &dim, K, &dim, &info );
@@ -306,7 +324,7 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 		for( iter = 0; iter < mc_iter; iter++ ) 
 		{
 			for( i = 0; i < dim; i++ )
-				psi[i * dim + i] = sqrt( Rf_rgamma( ( b_c + nu[i] ) / 2.0, 2.0 ) );
+				psi[ i * dim + i ] = sqrt( Rf_rgamma( ( b_c + nu[ i ] ) / 2.0, 2.0 ) );
 				//psi[i * dim + i] = sqrt( rchisq( b_c + nu[i] ) );
 
 			for( i = 0; i < dim - 1; i++ ) 
@@ -314,7 +332,7 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 				{
 					ij = j * dim + i;
 					//if( G[ij] == 1 ) psi[ij] = rnorm( 0, 1 ); else psi[ij] = 0.0;
-					psi[ij] = ( G[ij] == 1 ) ? norm_rand() : 0.0;
+					psi[ ij ] = ( G[ ij ] == 1 ) ? norm_rand() : 0.0;
 				}
 			
 			for( i = 0; i < dim - 1; i++ ) 
@@ -322,9 +340,9 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 				{
 					ij = j * dim + i;
 					
-					if( G[ij] == 0 )
+					if( G[ ij ] == 0 )
 					{
-						psi[ij] = 0.0; // it's not necessary
+						psi[ ij ] = 0.0; // it's not necessary
 						if( i > 0 )  
 						{
 							sumPsi = 0.0;
@@ -337,19 +355,19 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 							}
 							
 							//psi[i, j] <- - sum( psi[ 1 : ( i - 1 ), i ] * psi[ 1 : ( i - 1 ), j ] ) / psi[i, i]
-							psi[ij] = - sumPsi / psi[ i * dim + i ];
+							psi[ ij ] = - sumPsi / psi[ i * dim + i ];
 						}
 						
-						if( psi[ij] == R_PosInf ) psi[ij] = max_numeric_limits_ld;	
-						if( psi[ij] == R_NegInf ) psi[ij] = min_numeric_limits_ld;	
+						if( psi[ ij ] == R_PosInf ) psi[ ij ] = max_numeric_limits_ld;	
+						if( psi[ ij ] == R_NegInf ) psi[ ij ] = min_numeric_limits_ld;	
 												
 						//f_T[k] <- f_T[k] + psi[i, j] ^ 2
-						f_T[iter] += ( psi[ij] * psi[ij] ); 
+						f_T[ iter ] += ( psi[ ij ] * psi[ ij ] ); 
 					}
 				}
 		
 			// checking Inf values
-			if( f_T[iter] == R_PosInf ) f_T[iter] = max_numeric_limits_ld;			
+			if( f_T[ iter ] == R_PosInf ) f_T[ iter ] = max_numeric_limits_ld;			
 		} 
 	}
 	else
@@ -357,7 +375,7 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 		for( iter = 0; iter < mc_iter; iter++ ) 
 		{
 			for( i = 0; i < dim; i++ )
-				psi[i * dim + i] = sqrt( Rf_rgamma( ( b_c + nu[i] ) / 2.0, 2.0 ) );
+				psi[ i * dim + i ] = sqrt( Rf_rgamma( ( b_c + nu[ i ] ) / 2.0, 2.0 ) );
 				//psi[i * dim + i] = sqrt( rchisq( b_c + nu[i] ) );
 
 			for( i = 0; i < dim - 1; i++ ) 
@@ -365,7 +383,7 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 				{
 					ij = j * dim + i;
 					//if( G[ij] == 1 ) psi[ij] = rnorm( 0, 1 ); elsepsi[ij] = 0.0;
-					psi[ij] = ( G[ij] == 1 ) ? norm_rand() : 0.0;
+					psi[ ij ] = ( G[ ij ] == 1 ) ? norm_rand() : 0.0;
 				}
 			
 			for( i = 0; i < dim - 1; i++ ) 
@@ -373,7 +391,7 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 				{
 					ij = j * dim + i;
 					
-					if( G[ij] == 0 )
+					if( G[ ij ] == 0 )
 					{
 						//psi[i, j] = - sum( psi[ i, i : ( j - 1 ) ] * H[ i : ( j - 1 ), j ] )
 						sumPsiH = 0.0;
@@ -383,7 +401,7 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 							if( sumPsiH == R_NegInf ) sumPsiH = min_numeric_limits_ld;	
 							sumPsiH += ( psi[ h * dim + i ] * H[ j * dim + h ] ); 
 						}
-						psi[ij] = - sumPsiH;
+						psi[ ij ] = - sumPsiH;
 						
 						if( i > 0 )  //if( i > 1 )
 							for( r = 0; r < i; r++ ) //for( r in 1 : ( i - 1 ) )
@@ -403,19 +421,19 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 									sumPsiHj += ( psi[ h * dim + r ] * H[ j * dim + h ] );
 								
 								//psi[i, j] <- psi[i, j] - ( ( sum( psi[ r, r : i ] * H[ r : i, i ] ) ) * ( sum( psi[ r, r : j ] * H[ r : j, j ] ) ) ) / ( psi[i, i] )
-								psi[ij] -= ( sumPsiHi * sumPsiHj ) / psi[i * dim + i];
+								psi[ ij ] -= ( sumPsiHi * sumPsiHj ) / psi[ i * dim + i ];
 							}
 
-						if( psi[ij] == R_PosInf ) psi[ij] = max_numeric_limits_ld;	
-						if( psi[ij] == R_NegInf ) psi[ij] = min_numeric_limits_ld;	
+						if( psi[ ij ] == R_PosInf ) psi[ ij ] = max_numeric_limits_ld;	
+						if( psi[ ij ] == R_NegInf ) psi[ ij ] = min_numeric_limits_ld;	
 
 						//f_T[k] <- f_T[k] + psi[i, j] ^ 2
-						f_T[iter] += ( psi[ij] * psi[ij] ); 
+						f_T[ iter ] += ( psi[ ij ] * psi[ ij ] ); 
 					}
 				}
 		
 			// checking Inf values
-			if( f_T[iter] == R_PosInf ) f_T[iter] = max_numeric_limits_ld;			
+			if( f_T[ iter ] == R_PosInf ) f_T[ iter ] = max_numeric_limits_ld;			
 		}
 	}
 	PutRNGstate();	
